/* this source was algorythmically generated by makeswap.c */

#include "3dstype.h"
#include "swapbyte.h"
void N2ILong3ds(void *native, void *intel){
   *((unsigned long *)(intel)) = *((unsigned long *)(native));
}

void I2NLong3ds(void *intel, void *native){
   *((unsigned long *)(native)) = *((unsigned long *)(intel));
}

void N2IShort3ds(void *native, void *intel){
   *((unsigned short *)(intel)) = *((unsigned short *)(native));
}

void I2NShort3ds(void *intel, void *native){
   *((unsigned short *)(native)) = *((unsigned short *)(intel));
}

void N2IFloat3ds(void *native, void *intel){
   *((float *)(intel)) = *((float *)(native));
}

void I2NFloat3ds(void *intel, void *native){
   *((float *)(native)) = *((float *)(intel));
}

void N2IDouble3ds(void *native, void *intel){
   *((double *)(intel)) = *((double *)(native));
}

void I2NDouble3ds(void *intel, void *native){
   *((double *)(native)) = *((double *)(intel));
}

