//{{NO_DEPENDENCIES}}
// Microsoft Developer Studio generated include file.
// Used by LeoCAD.rc
//
#define IDAPPLY                         3
#define IDC_HTMDLG_IMAGEOPTIONS         3
#define ID_EDITOR_PREV                  3
#define ID_EDITOR_FLIP                  4
#define IDC_EDITGRP_NEWGROUP            4
#define ID_EDITOR_NEXT                  5
#define IDD_ABOUTBOX                    100
#define ID_INDICATOR_POSITION           102
#define IDD_PROPGENERAL                 103
#define ID_INDICATOR_STEP               103
#define IDD_PROPSUMMARY                 104
#define ID_INDICATOR_SNAP               104
#define IDD_PROPPIECES                  105
#define IDW_PIECEPREVIEW                105
#define IDW_PIECESLIST                  106
#define IDW_GROUPSBAR                   107
#define IDW_PIECESCOMBO                 108
#define IDW_COLORSLIST                  109
#define IDW_PIECEBAR_SPLITTER           110
#define IDW_PIECESTREE                  111
#define IDS_CANNOT_OPEN_CLIPBOARD       111
#define IDS_CANNOT_GET_CLIPBOARD_DATA   112
#define IDS_CANCEL_PROMPT               113
#define IDS_FAIL_ALLOC                  114
#define IDS_PROPSHT_CAPTION             115
#define IDD_PREFGENERAL                 116
#define IDS_OPENFILE_ERROR              116
#define IDD_PREFDRAWING                 117
#define IDD_PREFDETAIL                  118
#define IDD_PREFSCENE                   119
#define IDD_PREFPRINT                   120
#define IDR_MAINFRAME                   128
#define IDR_POPUPS                      130
#define IDC_VSPLITBAR                   131
#define IDR_TOOLSBAR                    132
#define IDC_ZOOM                        134
#define IDC_BRICK                       135
#define IDC_CAMERA                      136
#define IDC_ERASER                      137
#define IDC_SPOTLIGHT                   138
#define IDC_MOVE                        139
#define IDC_PAINT                       140
#define IDC_PAN                         141
#define IDC_ANGLE                       143
#define IDR_ANIMATORBAR                 144
#define IDC_ROTATE                      146
#define IDC_LIGHT                       147
#define IDC_ZOOM_REGION                 148
#define IDC_SELECT_GROUP                149
#define IDC_SELECT                      150
#define IDC_ROLL                        151
#define IDD_SELECT_OBJECTS              152
#define IDD_ARRAY                       153
#define IDD_STEP                        154
#define IDB_VIEWPORTS                   155
#define IDB_PIECEBAR                    156
#define IDD_IMAGE                       158
#define IDD_HTML                        159
#define IDD_PROGRESS                    161
#define IDR_PARTFRAME                   162
#define IDB_AUTOPAN                     164
#define IDC_PAN_UP                      165
#define IDC_PAN_LEFT                    166
#define IDC_PAN_DOWN                    167
#define IDC_PAN_RIGHT                   168
#define IDC_PAN_ALL                     169
#define IDD_GROUP                       170
#define IDB_HELP                        172
#define IDB_HOME                        173
#define IDB_MAIL                        174
#define IDB_INFO                        175
#define IDS_COLOR01                     176
#define IDS_COLOR02                     177
#define IDB_PARTICONS                   177
#define IDD_OPENDLG_TEMPLATE            177
#define IDS_COLOR03                     178
#define IDD_MODIFY                      178
#define IDS_COLOR04                     179
#define IDS_COLOR05                     180
#define IDS_COLOR06                     181
#define IDS_COLOR07                     182
#define IDS_COLOR08                     183
#define IDS_COLOR09                     184
#define IDB_PIECE                       184
#define IDS_COLOR10                     185
#define IDB_CAMERA                      185
#define IDS_COLOR11                     186
#define IDB_LIGHT                       186
#define IDS_COLOR12                     187
#define IDS_COLOR13                     188
#define IDS_COLOR14                     189
#define IDB_ZOOMIN                      189
#define IDS_COLOR15                     190
#define IDB_ZOOMOUT                     190
#define IDS_COLOR16                     191
#define IDB_GROUP                       191
#define IDS_COLOR17                     192
#define IDB_UNGROUP                     192
#define IDS_COLOR18                     193
#define IDD_MINIFIG                     193
#define IDR_FULLSCREEN                  193
#define IDS_COLOR19                     194
#define IDD_SAVEDLG_TEMPLATE            194
#define IDS_COLOR20                     195
#define IDB_FULLSCREEN                  195
#define IDS_COLOR21                     196
#define IDS_COLOR22                     197
#define IDB_PHOTO                       197
#define IDS_COLOR23                     198
#define IDD_EXPORTPOV                   198
#define IDB_DELETE                      198
#define IDS_COLOR24                     199
#define IDB_PREFERENCES                 199
#define IDD_SAVEPICTUREDLG_TEMPLATE     199
#define IDS_COLOR25                     200
#define IDS_COLOR26                     201
#define IDC_PAN_NE                      201
#define IDS_COLOR27                     202
#define IDC_PAN_NW                      202
#define IDS_COLOR28                     203
#define IDC_PAN_SE                      203
#define IDC_PAN_SW                      204
#define IDR_PIECEEDITOR                 205
#define IDD_PIECEEDITOR                 208
#define IDB_EDITOR_ICONS                209
#define IDD_LIBRARY                     211
#define IDR_LIBRARY                     213
#define IDD_TERRAIN                     216
#define IDR_TERRAIN                     217
#define IDD_TERRAIN_OPTIONS             218
#define IDR_PREVIEW                     221
#define IDD_EDIT_GROUPS                 231
#define IDD_LIBRARY_TEXTURES            234
#define IDD_PREFKEYBOARD                235
#define IDD_TRANSFORM                   236
#define IDD_CATEGORY                    237
#define IDC_ROTX                        238
#define IDC_ROTY                        239
#define IDC_SELDLG_LIST                 1000
#define IDC_SELDLG_ALL                  1001
#define IDC_SELDLG_NONE                 1002
#define IDC_SELDLG_INVERT               1003
#define IDC_SELDLG_NAME                 1004
#define IDC_SELDLG_TYPE                 1005
#define IDC_SELDLG_PIECES               1007
#define IDC_SELDLG_CAMERAS              1008
#define IDC_SELDLG_LIGHTS               1009
#define IDC_SELDLG_GROUPS               1010
#define IDC_SELDLG_EDIT                 1012
#define IDC_ARRAY_1D                    1013
#define IDC_ARRAY_2D                    1014
#define IDC_ARRAY_3D                    1015
#define IDC_ARRAY_1D_COUNT              1016
#define IDC_ARRAY_1D_COUNT_SPIN         1017
#define IDC_ARRAY_2D_COUNT              1018
#define IDC_ARRAY_2D_COUNT_SPIN         1019
#define IDC_ARRAY_2D_X                  1020
#define IDC_ARRAY_2D_Y                  1021
#define IDC_ARRAY_2D_Z                  1022
#define IDC_ARRAY_3D_COUNT              1023
#define IDC_ARRAY_3D_COUNT_SPIN         1024
#define IDC_ARRAY_3D_X                  1025
#define IDC_ARRAY_3D_Y                  1026
#define IDC_ARRAY_3D_Z                  1027
#define IDC_ARRAY_MOVE_X                1028
#define IDC_ARRAY_ROTATE_X              1029
#define IDC_ARRAY_MOVE_Y                1030
#define IDC_ARRAY_ROTATE_Y              1031
#define IDC_ARRAY_MOVE_Z                1032
#define IDC_ARRAY_ROTATE_Z              1033
#define IDC_ARRAY_2D_X_SPIN             1034
#define IDC_ARRAY_2D_Y_SPIN             1035
#define IDC_ARRAY_2D_Z_SPIN             1036
#define IDC_ARRAY_3D_X_SPIN             1037
#define IDC_ARRAY_3D_Y_SPIN             1038
#define IDC_ARRAY_3D_Z_SPIN             1039
#define IDC_ARRAY_MOVE_X_SPIN           1040
#define IDC_ARRAY_ROTATE_X_SPIN         1041
#define IDC_ARRAY_MOVE_Y_SPIN           1042
#define IDC_ARRAY_ROTATE_Y_SPIN         1043
#define IDC_ARRAY_MOVE_Z_SPIN           1044
#define IDC_ARRAY_ROTATE_Z_SPIN         1045
#define IDC_ARRAY_TOTAL                 1046
#define IDC_STEP_EDIT                   1047
#define IDC_STEP_SLIDER                 1048
#define IDC_STEP_SPIN                   1049
#define IDC_IMGDLG_WIDTH                1050
#define IDC_IMGDLG_HEIGHT               1051
#define IDC_IMGDLG_QUALITY              1052
#define IDC_IMGDLG_PROGRESSIVE          1054
#define IDC_HTMDLG_SINGLEPAGE           1055
#define IDC_HTMDLG_ONESTEP              1056
#define IDC_IMGDLG_TRANSPARENT          1057
#define IDC_IMGDLG_GIF                  1058
#define IDC_HTMDLG_OUTPUT               1058
#define IDC_HTMDLG_LISTIMAGES           1062
#define IDC_HTMDLG_INDEX                1064
#define IDC_HTMDLG_BROWSEFOLDER         1067
#define IDC_PRGDLG_PROGRESS             1068
#define IDC_PRGDLG_TEXT                 1069
#define IDC_PRGDLG_TEXT2                1070
#define IDC_GRPDLG_NAME                 1071
#define IDC_PROP_PIECES_LIST            1072
#define IDC_ABTDLG_INFO                 1073
#define IDC_GENDLG_NUMBERS              1074
#define IDC_MODDLG_HIDDEN               1074
#define IDC_SAVEDLG_PREVIEW             1074
#define IDC_GENDLG_PREVIEW              1075
#define IDC_AIDDLG_SNAPX                1076
#define IDC_GENDLG_ZOOM                 1076
#define IDC_IMGDLG_HIGHCOLOR            1076
#define IDC_SCNDLG_IMAGEBG              1076
#define IDC_GENDLG_GROUP                1077
#define IDC_SCNDLG_FLOORSKY             1077
#define IDC_SCNDLG_TERRAIN              1077
#define IDC_AIDDLG_SNAPX2               1077
#define IDC_GENDLG_SUBPARTS             1078
#define IDC_SCNDLG_SKYGRADIENT          1078
#define IDC_GENDLG_AUTOSAVE             1079
#define IDC_AIDDLG_FIXEDKEYS            1080
#define IDC_GENDLG_COMBO                1080
#define IDC_AIDDLG_PREVIEW              1081
#define IDC_GENDLG_BACKUP               1081
#define IDC_AIDDLG_COLLISION            1082
#define IDC_AIDDLG_MOVE                 1083
#define IDC_AIDDLG_LOCKX                1084
#define IDC_AIDDLG_LOCKY                1085
#define IDC_DETDLG_LINEAR               1086
#define IDC_HTMDLG_LIST_STEP            1087
#define IDC_DETDLG_FAST                 1087
#define IDC_PROP_GEN_LOCATION           1088
#define IDC_HTMDLG_LIST_END             1088
#define IDC_DETDLG_SOLID                1088
#define IDC_PROP_SUM_AUTHOR             1089
#define IDC_AIDDLG_LOCKZ                1089
#define IDC_DETDLG_NOALPHA              1089
#define IDC_PROP_GEN_DOSNAME            1090
#define IDC_PROP_SUM_DESCRIPTION        1090
#define IDC_DETDLG_EDGES                1090
#define IDC_AIDDLG_CENTIMETERS          1090
#define IDC_PROP_GEN_SIZE               1091
#define IDC_PROP_SUM_COMMENTS           1091
#define IDC_DETDLG_DITHER               1091
#define IDC_AIDDLG_3DMOUSE              1091
#define IDC_AIDDLG_GLOBAL               1091
#define IDC_PROP_GEN_CREATED            1092
#define IDC_DETDLG_LIGHTING             1092
#define IDC_PROP_GEN_MODIFIED           1093
#define IDC_DETDLG_SMOOTH               1093
#define IDC_PROP_GEN_ACCESSED           1094
#define IDC_DETDLG_ANTIALIAS            1094
#define IDC_AIDDLG_ANGLE                1095
#define IDC_GENDLG_SAVETIME             1095
#define IDC_MODDLG_ROTX                 1095
#define IDC_EDTDLG_CONNECTION           1095
#define IDC_DETDLG_LINE                 1096
#define IDC_PRNDLG_MARGIN_TOP           1096
#define IDC_AIDDLG_GRIDSIZE             1096
#define IDC_MODDLG_ROTY                 1096
#define IDC_DETDLG_HIDDEN               1097
#define IDC_PRNDLG_MARGIN_BOTTOM        1097
#define IDC_MODDLG_ROTZ                 1097
#define IDC_DETDLG_BACKGROUND           1098
#define IDC_PRNDLG_MARGIN_RIGHT         1098
#define IDC_MODDLG_POSX                 1098
#define IDC_MODDLG_POSY                 1099
#define IDC_MODDLG_POSZ                 1100
#define IDC_OPENDLG_PREVIEW             1102
#define IDC_OPENDLG_TEXT                1103
#define IDC_SCNDLG_BGIMAGE_BROWSE       1104
#define IDC_SCNDLG_FOGCOLOR             1105
#define IDC_SCNDLG_BGCOLOR              1106
#define IDC_SCNDLG_AMBIENTLIGHT         1107
#define IDC_SCNDLG_BGIMAGE              1109
#define IDC_SCNDLG_BGTILE               1110
#define IDC_SCNDLG_FOG                  1111
#define IDC_SCNDLG_FOGDENSITY           1112
#define IDC_PRNDLG_HEADER               1113
#define IDC_PRNDLG_FOOTER               1114
#define IDC_PRNDLG_HEADERBTN            1115
#define IDC_PRNDLG_FOOTERBTN            1116
#define IDC_PRNDLG_INST_ROWS            1117
#define IDC_PRNDLG_INST_COLS            1118
#define stc32                           0x045f
#define IDC_PRNDLG_CAT_COLS             1119
#define IDC_PRNDLG_CAT_ROWS             1120
#define IDC_PRNDLG_NUMBERS              1121
#define IDC_PRNDLG_BORDER               1122
#define IDC_PRNDLG_MARGIN_LEFT          1123
#define IDC_AIDDLG_GRID                 1125
#define IDC_AIDDLG_AXIS                 1126
#define IDC_AIDDLG_SNAPY                1127
#define IDC_AIDDLG_SNAPZ                1128
#define IDC_AIDDLG_SNAPA                1129
#define IDC_GENDLG_FOLDER               1130
#define IDC_GENDLG_FOLDERBTN            1131
#define IDC_GENDLG_MOUSE                1132
#define IDC_GENDLG_GAMMA                1133
#define IDC_GENDLG_USER                 1133
#define IDC_MODDLG_LIST                 1136
#define IDC_MODDLG_COLOR                1137
#define IDC_MODDLG_PIECE                1142
#define IDC_MODDLG_ROTATION             1145
#define IDC_MODDLG_FROM                 1146
#define IDC_EDTDLG_DESCRIPTION          1146
#define IDC_MODDLG_TO                   1147
#define IDC_EDTDLG_NAME                 1147
#define IDC_MODDLG_ROTATION2            1148
#define IDC_IMGDLG_PAUSE                1148
#define IDC_MODDLG_STEPFROM             1148
#define IDC_POVDLG_LGEO                 1148
#define IDC_POVDLG_POVRAY               1149
#define IDC_MODDLG_FOV                  1149
#define IDC_MODDLG_OK                   1150
#define IDC_POVDLG_RENDER               1150
#define IDC_MODDLG_UPY                  1150
#define IDC_MODDLG_CLOSE                1151
#define IDC_POVDLG_LGEOBROWSE           1151
#define IDC_MODDLG_UPX                  1151
#define IDC_MODDLG_APPLY                1152
#define IDC_POVDLG_POVBROWSE            1152
#define IDC_POVDLG_OUTPOV               1153
#define IDC_MODDLG_UPZ                  1153
#define IDC_IMGDLG_SINGLE               1154
#define IDC_POVDLG_POVOUTBROWSE         1154
#define IDC_MODDLG_NEAR                 1154
#define IDC_IMGDLG_MULTIPLE             1155
#define IDC_MODDLG_FAR                  1155
#define IDC_IMGDLG_BMP                  1158
#define IDC_IMGDLG_JPG                  1160
#define IDC_IMGDLG_AVI                  1161
#define IDC_IMGDLG_FROM                 1162
#define IDC_IMGDLG_TO                   1163
#define IDC_SAVEPICTURE_OPTIONS         1164
#define IDC_IMGDLG_PNG                  1164
#define IDC_MODDLG_STEPTO               1165
#define IDC_MODDLG_FOVSTATIC            1166
#define IDC_EDITORSTATIC                1167
#define IDC_EDTDLG_LIST                 1168
#define IDC_BUTTON2                     1170
#define ID_EDITOR_GUESS                 1171
#define IDC_EDTDLG_VERT                 1177
#define IDC_EDTDLG_HORZ                 1178
#define IDC_SCNDLG_SKYCOLOR1            1185
#define IDC_LIBDLG_TREE                 1185
#define IDC_SCNDLG_GRAD1                1185
#define IDC_SCNDLG_SKYCOLOR2            1186
#define IDC_LIBDLG_LIST                 1186
#define IDC_SCNDLG_GRAD2                1186
#define IDC_MF_HATCOLOR                 1189
#define IDC_MF_HEADCOLOR                1190
#define IDC_HTMDLG_HIGHLIGHT            1190
#define IDC_MF_TORSOCOLOR               1191
#define IDC_GRID                        1191
#define IDC_MF_NECKCOLOR                1192
#define IDC_MF_ARMLCOLOR                1193
#define IDC_MF_ARMRCOLOR                1194
#define IDC_MF_HANDLCOLOR               1195
#define IDC_MF_HANDRCOLOR               1196
#define IDC_MF_TOOLLCOLOR               1197
#define IDC_MF_TOOLRCOLOR               1198
#define IDC_MF_HIPSCOLOR                1199
#define IDC_MF_LEGLCOLOR                1200
#define IDC_TEROPT_XPAT                 1200
#define IDC_MF_LEGRCOLOR                1201
#define IDC_TEROPT_YPAT                 1201
#define IDC_MF_SHOELCOLOR               1202
#define IDC_TEROPT_XSIZE                1202
#define IDC_MF_SHOERCOLOR               1203
#define IDC_TEROPT_YSIZE                1203
#define IDC_MF_HAT                      1204
#define IDC_TEROPT_COLOR                1204
#define IDC_MF_HEAD                     1205
#define IDC_TEROPT_TEXTURENAME          1205
#define IDC_MF_TORSO                    1206
#define IDC_TEROPT_TEXTURE              1206
#define IDC_MF_NECK                     1207
#define IDC_TEROPT_TEXTUREBROWSE        1207
#define IDC_MF_ARML                     1208
#define IDC_TEROPT_SMOOTH               1208
#define IDC_MF_ARMR                     1209
#define IDC_MF_HANDL                    1210
#define IDC_EDIT1                       1210
#define IDC_MF_HANDR                    1211
#define IDC_TEROPT_FLAT                 1211
#define IDC_TRANSDLG_GY                 1211
#define IDC_CATDLG_KEYWORDS             1211
#define IDC_MF_TOOLL                    1212
#define IDC_EDIT2                       1212
#define IDC_SCNDLG_SOLID                1212
#define IDC_TRANSDLG_GZ                 1212
#define IDC_MF_TOOLR                    1213
#define IDC_SCNDLG_GRADIENT             1213
#define IDC_TRANSDLG_OY                 1213
#define IDC_MF_HIPS                     1214
#define IDC_SCNDLG_IMAGE                1214
#define IDC_TREE                        1214
#define IDC_TRANSDLG_OZ                 1214
#define IDC_MF_LEGL                     1215
#define IDC_MODDLG_PLANESSTATIC         1215
#define IDC_MF_LEGR                     1216
#define IDC_MF_SHOEL                    1217
#define IDC_MF_SHOER                    1218
#define IDC_PREVIEWSTATIC               1219
#define IDC_MF_HATANGLE                 1220
#define IDC_MF_HEADANGLE                1221
#define IDC_MF_NECKANGLE                1222
#define IDC_MF_ARMLANGLE                1223
#define IDC_HTMLDLG_HTMLEXT             1223
#define IDC_MF_ARMRANGLE                1224
#define IDC_MF_HANDLANGLE               1225
#define ID_LIBTEX_ADD                   1225
#define IDC_MF_HANDRANGLE               1226
#define ID_LIBTEX_REMOVE                1226
#define IDC_MF_TOOLLANGLE               1227
#define IDC_MF_TOOLRANGLE               1228
#define ID_LIBTEX_LIST                  1228
#define IDC_MF_LEGLANGLE                1229
#define IDC_ABTDLG_HOMEPAGE             1229
#define IDC_MF_LEGRANGLE                1230
#define IDC_KEYDLG_CMDLIST              1230
#define IDC_MF_SHOELANGLE               1231
#define IDC_MF_SHOERANGLE               1232
#define IDC_MF_HATSPIN                  1233
#define IDC_KEYDLG_REMOVE               1233
#define IDC_MF_HEADSPIN                 1234
#define IDC_MF_NECKSPIN                 1235
#define IDC_MF_ARMLSPIN                 1236
#define IDC_KEYDLG_RESET                1236
#define IDC_MF_ARMRSPIN                 1237
#define IDC_KEYDLG_KEYEDIT              1237
#define IDC_MF_HANDLSPIN                1238
#define IDC_KEYDLG_ASSIGN               1238
#define IDC_MF_HANDRSPIN                1239
#define IDC_KEYDLG_COMBO                1239
#define IDC_MF_TOOLLSPIN                1240
#define IDC_KEYDLG_SAVE                 1240
#define IDC_MF_TOOLRSPIN                1241
#define IDC_KEYDLG_LOAD                 1241
#define IDC_MF_LEGLSPIN                 1242
#define IDC_KEYDLG_FILENAME             1242
#define IDC_MF_LEGRSPIN                 1243
#define IDC_ABTDLG_VERSION              1243
#define IDC_MF_SHOELSPIN                1244
#define IDC_GENDLG_UPDATES              1244
#define IDC_MF_SHOERSPIN                1245
#define IDC_TRANSDLG_GX                 1245
#define IDC_TRANSDLG_OX                 1246
#define IDC_CATDLG_NAME                 1246
#define ID_EDIT_SELECTALL               32772
#define ID_EDIT_SELECTNONE              32773
#define ID_EDIT_SELECTINVERT            32774
#define ID_EDIT_SELECTBYNAME            32775
#define ID_FILE_MERGE                   32776
#define ID_FILE_PROPERTIES              32777
#define ID_VIEW_FULLSCREEN              32780
#define ID_VIEW_STEP_NEXT               32782
#define ID_VIEW_STEP_PREVIOUS           32783
#define ID_VIEW_STEP_CHOOSE             32784
#define ID_VIEW_STEP_INSERT             32785
#define ID_ACTION_SELECT                32789
#define ID_ACTION_INSERT                32790
#define ID_ACTION_LIGHT                 32791
#define ID_ACTION_SPOTLIGHT             32792
#define ID_ACTION_CAMERA                32793
#define ID_ACTION_MOVE                  32794
#define ID_ACTION_ROTATE                32795
#define ID_ACTION_REMOVE                32796
#define ID_ACTION_PAINT                 32797
#define ID_ACTION_ZOOM                  32798
#define ID_ACTION_ZOOM_REGION           32799
#define ID_ACTION_PAN                   32800
#define ID_ACTION_ROTATE_VIEW           32801
#define ID_ACTION_ROLL                  32802
#define ID_ACTION_ZOOM_EXTENTS          32803
#define ID_ZOOM_EXTENTS                 32807
#define ID_PIECE_GROUP                  32808
#define ID_PIECE_UNGROUP                32809
#define ID_VIEW_PREFERENCES             32810
#define ID_ANIMATOR_PLAY                32814
#define ID_ANIMATOR_STOP                32815
#define ID_RENDER_BACKGROUND            32820
#define ID_RENDER_BOX                   32821
#define ID_PIECE_INSERT                 32822
#define ID_PIECE_DELETE                 32823
#define ID_PIECE_MODIFY                 32824
#define ID_VIEW_STEP_FIRST              32825
#define ID_VIEW_STEP_LAST               32826
#define ID_PIECE_HIDESELECTED           32827
#define ID_PIECE_HIDEUNSELECTED         32828
#define ID_PIECE_UNHIDEALL              32829
#define ID_ANIMATOR_TOGGLE              32830
#define ID_ANIMATOR_KEY                 32831
#define ID_FILE_EXPORT_WAVEFRONT        32833
#define ID_FILE_EXPORT_LDRAW            32834
#define ID_FILE_EXPORT_POVRAY           32835
#define ID_PIECEBAR_NUMBERS             32839
#define ID_PIECEBAR_SUBPARTS            32841
#define ID_PIECE_ARRAY                  32842
#define ID_PIECE_MINIFIGWIZARD          32843
#define ID_FILE_EXPORT_3DSTUDIO         32844
#define ID_FILE_EXPORT_HTML             32845
#define ID_HELP_LEOCADHOMEPAGE          32846
#define ID_HELP_CHECKFORUPDATES         32847
#define ID_HELP_SENDEMAIL               32848
#define ID_FILE_CHANGEBITMAP            32864
#define ID_FILE_EDITPIECELIBRARY        32865
#define ID_VIEW_ZOOMIN                  32866
#define ID_VIEW_ZOOMOUT                 32867
#define ID_PIECE_DETACH                 32868
#define ID_PIECE_ATTACH                 32869
#define ID_PIECE_EDITGROUP              32870
#define ID_PIECE_EDITGROUPS             32871
#define ID_VIEW_CAMERAS_DUMMY           32872
#define ID_PRINT_FILENAME               32873
#define ID_PRINT_AUTHOR                 32874
#define ID_PRINT_DESCRIPTION            32875
#define ID_PRINT_PAGENUMBER             32876
#define ID_PRINT_TOTALPAGES             32877
#define ID_PRINT_CURRENTTIME            32878
#define ID_PRINT_CURRENTDATE            32879
#define ID_PRINT_LEFTALIGN              32880
#define ID_PRINT_CENTER                 32881
#define ID_PRINT_RIGHTALIGN             32882
#define ID_SNAP_ON                      32883
#define ID_SNAP_ANGLE                   32884
#define ID_MODDLG_PIECES                32884
#define ID_MODDLG_CAMERAS               32885
#define ID_MODDLG_LIGHTS                32886
#define ID_FILE_PRINTCATALOG            32887
#define ID_LIBDLG_FILE_PRINTCATALOG     32887
#define ID_FILE_MERGEUPDATE             32888
#define ID_LIBDLG_FILE_MERGEUPDATE      32888
#define ID_VIEWPORT01                   32891
#define ID_VIEWPORT02                   32892
#define ID_VIEWPORT03                   32893
#define ID_VIEWPORT04                   32894
#define ID_VIEWPORT05                   32895
#define ID_VIEWPORT06                   32896
#define ID_VIEWPORT07                   32897
#define ID_VIEWPORT08                   32898
#define ID_VIEWPORT09                   32899
#define ID_VIEWPORT10                   32900
#define ID_VIEWPORT11                   32901
#define ID_VIEWPORT12                   32902
#define ID_VIEWPORT13                   32903
#define ID_VIEWPORT14                   32904
#define ID_PIECE_GROUP01                32911
#define ID_PIECE_GROUP02                32912
#define ID_PIECE_GROUP03                32913
#define ID_PIECE_GROUP04                32914
#define ID_PIECE_GROUP05                32915
#define ID_PIECE_GROUP06                32916
#define ID_PIECE_GROUP07                32917
#define ID_PIECE_GROUP08                32918
#define ID_PIECE_GROUP09                32919
#define ID_PIECE_GROUP10                32920
#define ID_PIECE_GROUP11                32921
#define ID_PIECE_GROUP12                32922
#define ID_PIECE_GROUP13                32923
#define ID_PIECE_GROUP14                32924
#define ID_PIECE_GROUP15                32925
#define ID_PIECE_GROUP16                32926
#define ID_PIECE_GROUP17                32927
#define ID_PIECE_GROUP18                32928
#define ID_PIECE_GROUP19                32929
#define ID_PIECE_GROUP20                32930
#define ID_PIECE_GROUP21                32931
#define ID_PIECE_GROUP22                32932
#define ID_PIECE_GROUP23                32933
#define ID_PIECE_GROUP24                32934
#define ID_PIECE_GROUP25                32935
#define ID_PIECE_GROUP26                32936
#define ID_PIECE_GROUP27                32937
#define ID_PIECE_GROUP28                32938
#define ID_PIECE_GROUP29                32939
#define ID_PIECE_GROUP30                32940
#define ID_PIECE_GROUP31                32941
#define ID_PIECE_GROUP32                32942
#define ID_CAMERA_FIRST                 32943
#define ID_CAMERA_LAST                  33011
#define ID_SNAP_SNAPX                   33012
#define ID_SNAP_SNAPY                   33013
#define ID_SNAP_SNAPZ                   33014
#define ID_SNAP_SNAPALL                 33015
#define ID_SNAP_SNAPNONE                33016
#define ID_FILE_SAVEPICTURE             33017
#define ID_FILE_PRINTPIECELIST          33018
#define ID_EDITOR_FILE_OPEN             33027
#define ID_EDITOR_FILE_SAVE             33028
#define ID_EDITOR_FILE_RETURN           33029
#define ID_EDITOR_VIEW_BACKFACES        33030
#define ID_EDITOR_VIEW_BACKGROUND       33032
#define ID_EDITOR_INLINE                33033
#define ID_EDITOR_CONNECTION            33034
#define ID_EDITOR_VIEW_CONNECTIONS      33035
#define ID_EDITOR_ADD                   33036
#define ID_EDITOR_FILE_NEW              33038
#define ID_PIECE_COPYKEYS               33039
#define ID_LIBDLG_GROUP_INSERT          33053
#define ID_LIBDLG_GROUP_DELETE          33054
#define ID_LIBDLG_GROUP_RENAME          33055
#define ID_LIBDLG_GROUP_MOVEUP          33056
#define ID_LIBDLG_GROUP_MOVEDOWN        33057
#define ID_LIBDLG_FILE_RESET            33058
#define ID_LIBDLG_FILE_SAVEAS           33061
#define ID_LIBDLG_FILE_LOADBITMAP       33062
#define ID_LIBDLG_FILE_OPEN             33066
#define ID_LIBDLG_FILE_SAVE             33067
#define ID_LIBDLG_PIECE_EDIT            33068
#define ID_LIBDLG_PIECE_NEW             33069
#define ID_EDITOR_INLINE_ALL            33086
#define ID_LIBDLG_PIECE_DELETE          33087
#define ID_SNAP_0                       33088
#define ID_SNAP_1                       33089
#define ID_SNAP_2                       33090
#define ID_SNAP_3                       33091
#define ID_SNAP_4                       33092
#define ID_SNAP_5                       33093
#define ID_SNAP_6                       33094
#define ID_SNAP_7                       33095
#define ID_SNAP_8                       33096
#define ID_SNAP_9                       33097
#define ID_SNAP_10                      33098
#define ID_SNAP_11                      33099
#define ID_SNAP_12                      33100
#define ID_SNAP_13                      33101
#define ID_SNAP_14                      33102
#define ID_SNAP_15                      33103
#define ID_SNAP_16                      33104
#define ID_SNAP_17                      33105
#define ID_SNAP_18                      33106
#define ID_SNAP_19                      33107
#define ID_FILE_TERRAINEDITOR           33108
#define ID_TERDLG_FILE_SAVEAS           33112
#define ID_TERDLG_EDIT_RANDOM           33113
#define ID_TERDLG_FILE_OPEN             33114
#define ID_TERDLG_FILE_SAVE             33115
#define ID_TERDLG_EDIT_PREFERENCES      33116
#define ID_TERDLG_EDIT_SELECT           33119
#define ID_TERDLG_EDIT_ZOOM             33120
#define ID_TERDLG_EDIT_PAN              33121
#define ID_TERDLG_EDIT_ROTATE           33122
#define ID_TERDLG_EDIT_RESETCAMERA      33124
#define ID_TERDLG_HELP                  33125
#define ID_VIEW_CAMERAS_RESET           33126
#define ID_PREVIEW_ONEPAGE              33132
#define ID_PREVIEW_TWOPAGES             33133
#define ID_PREVIEW_PRINT                33134
#define ID_PREVIEW_PREV                 33135
#define ID_PREVIEW_NEXT                 33136
#define ID_PREVIEW_ZOOMIN               33137
#define ID_PREVIEW_ZOOMOUT              33138
#define ID_PREVIEW_CLOSE                33139
#define ID_PIECE_PREVIOUS               33140
#define ID_PIECE_NEXT                   33141
#define ID_LOCK_2BUTTONS                33143
#define ID_LOCK_3DMOVEMENT              33144
#define ID_LOCK_LOCKX                   33145
#define ID_LOCK_LOCKY                   33146
#define ID_LOCK_LOCKZ                   33147
#define ID_LOCK_UNLOCKALL               33148
#define ID_LOCK_ON                      33149
#define ID_PIECE_MIRROR                 33150
#define ID_FILE_IMPORTPIECE             33152
#define ID_VIEW_NEWVIEW                 33153
#define ID_VIEW_STEP_DELETE             33154
#define ID_LIBDLG_FILE_TEXTURES         33157
#define ID_PIECEBAR_NEWCATEGORY         33159
#define ID_PIECEBAR_EDITCATEGORY        33160
#define ID_PIECEBAR_REMOVECATEGORY      33161
#define ID_PIECE_TRANSFORM              33162
#define ID_LIBDLG_CATEGORY_NEW          33163
#define ID_LIBDLG_CATEGORY_REMOVE       33164
#define ID_LIBDLG_CATEGORY_EDIT         33165
#define ID_LIBDLG_CATEGORY_RESET        33166
#define ID_SNAP_XY                      33167
#define ID_SNAP_Z                       33168
#define ID_VIEW_PIECES_BAR              59425
#define ID_VIEW_TOOLS_BAR               59426
#define ID_VIEW_ANIMATION_BAR           59427
#define ID_VIEW_MODIFY_BAR              59428

// Next default values for new objects
// 
#ifdef APSTUDIO_INVOKED
#ifndef APSTUDIO_READONLY_SYMBOLS
#define _APS_3D_CONTROLS                     1
#define _APS_NEXT_RESOURCE_VALUE        240
#define _APS_NEXT_COMMAND_VALUE         33169
#define _APS_NEXT_CONTROL_VALUE         1247
#define _APS_NEXT_SYMED_VALUE           121
#endif
#endif
